﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Security;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        const string urlBase = "https://reader.datasymbol.com/api/";
        const string urlJob = "https://reader.datasymbol.com/api/job";
        const string apiKey = "1D7NFyPHcOuICR6nw0hq";                   //!!! use your own key

        //barcode types
        const uint ST_CODE128               = 0x00000001;
        const uint ST_CODE39                = 0x00000002;
        const uint ST_INTERL25              = 0x00000004;
        const uint ST_EAN13                 = 0x00000008;
        const uint ST_EAN8                  = 0x00000010;
        const uint ST_CODABAR               = 0x00000020;
        const uint ST_CODE11                = 0x00000040;
        const uint ST_UPCA                  = 0x00000080;
        const uint ST_UPCE                  = 0x00000100;
        const uint ST_INDUSTR25             = 0x00000200;
        const uint ST_CODE93                = 0x00000400;
        const uint ST_DATABAR_OMNI          = 0x00000800;
        const uint ST_DATABAR_LIM           = 0x00001000;
        const uint ST_DATABAR_STACKED       = 0x00002000;
        const uint ST_DATABAR_EXP           = 0x00004000;
        const uint ST_DATABAR_EXP_STACKED   = 0x00008000;
        const uint ST_LINEAR_UNDEC          = 0x01000000;
        const uint ST_PDF417_UNDEC          = 0x02000000;
        const uint ST_DATAMATRIX_UNDEC      = 0x04000000;
        const uint ST_QRCODE_UNDEC          = 0x08000000;
        const uint ST_AZTECCODE_UNDEC       = 0x00100000;
        const uint ST_DATAMATRIX            = 0x10000000;
        const uint ST_PDF417                = 0x20000000;
        const uint ST_QRCODE                = 0x40000000;
        const uint ST_AZTECCODE             = 0x80000000;
        const uint ST_LINEAR_SET            = (ST_CODE128 | ST_CODE39 | ST_INTERL25 | ST_EAN13 | ST_UPCA | ST_UPCE | ST_EAN8 | ST_CODABAR | ST_CODE11 | ST_INDUSTR25 | ST_CODE93 | ST_DATABAR_OMNI | ST_DATABAR_LIM | ST_DATABAR_STACKED | ST_DATABAR_EXP | ST_DATABAR_EXP_STACKED);

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //select file
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            if (openFileDialog1.ShowDialog() != DialogResult.OK)
                return;

            //draw the source image
            Image img = Image.FromFile(openFileDialog1.FileName, true);
            pictureBox1.Image = img;

            //decode file
            uint barcodeTypes = 0;
            if (checkBox1.CheckState == CheckState.Checked)
                barcodeTypes |= ST_LINEAR_SET;
            if (checkBox2.CheckState == CheckState.Checked)
                barcodeTypes |= ST_PDF417;
            if (checkBox3.CheckState == CheckState.Checked)
                barcodeTypes |= ST_DATAMATRIX;
            if (checkBox4.CheckState == CheckState.Checked)
                barcodeTypes |= ST_QRCODE;
            if (checkBox5.CheckState == CheckState.Checked)
                barcodeTypes |= ST_AZTECCODE;

            XmlNode nodeBarcodes = DecodeFile(openFileDialog1.FileName, barcodeTypes);

            //draw result
            DrawResult(img, nodeBarcodes);
        }

        private XmlNode DecodeFile(string fileName, uint barcodeTypes)
        {
            //try {

            textBox1.AppendText("\r\n***** [" + Path.GetFileName(fileName) + "] *****\r\n   --- sending job ...\r\n");

            //send job
            string xmlString = Upload(apiKey, urlJob, fileName, barcodeTypes).Result;
            //xml result
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xmlString);
            //check error code
            XmlNode nodeErr = doc.DocumentElement.SelectSingleNode("/BarcodeReader/error");
            if ( Int32.Parse(nodeErr.Attributes["code"]?.InnerText) != 1001)
            {
                textBox1.AppendText(nodeErr.InnerText + "\r\n");
                return null;
            }

            textBox1.AppendText("Ok\r\n");

            XmlNode nodeJob = doc.DocumentElement.SelectSingleNode("/BarcodeReader/job");
            int jobID = Int32.Parse(nodeJob.Attributes["id"]?.InnerText);
            int qNum = Int32.Parse(nodeJob.Attributes["queue"]?.InnerText);

            textBox1.AppendText("job ID: " + jobID.ToString() + ", queue: " + qNum.ToString() + "\r\n");

            //wait while processing
            Thread.Sleep(1000);

            //check job
            int err = 1;
            for (int i = 0; i < 3; ++i)
            {
                textBox1.AppendText("   --- checking result ...\r\n");
                string endpoint = urlBase + apiKey + "/" + jobID.ToString();
                xmlString = Query(apiKey, endpoint).Result;
                doc.LoadXml(xmlString);
                nodeErr = doc.DocumentElement.SelectSingleNode("/BarcodeReader/error");
                err = Int32.Parse(nodeErr.Attributes["code"]?.InnerText);
                if (!(err == 1001 || err == 1002))
                    break;
                Thread.Sleep(1000);
            }

            //error
            if (err != 0)
            {
                textBox1.AppendText(nodeErr.InnerText + "\r\n");
                return null;
            }

            textBox1.AppendText("Ok\r\n");

            XmlNode nodeBarcodes = doc.DocumentElement.SelectSingleNode("/BarcodeReader/result/barcodes");
            if (nodeBarcodes != null)
            {
                int barNum = Int32.Parse(nodeBarcodes.Attributes["qty"]?.InnerText);
                string outText="";
                for (int i = 0; i < barNum; ++i)
                {
                    XmlNode nodeBarcode = nodeBarcodes.ChildNodes.Item(i);
                    Debug.Assert(nodeBarcode != null && nodeBarcode.Name=="barcode");
                    string barcodeText;
                    if (nodeBarcode.SelectSingleNode("text")?.InnerText.Length != 0)
                        barcodeText = nodeBarcode.SelectSingleNode("text")?.InnerText;
                    else
                        barcodeText = "binary data";// Convert.FromBase64String(nodeBarcode.SelectSingleNode("data")?.InnerText);
                    outText += barcodeText + "|";
                }
                textBox1.AppendText("barcodes: " + barNum.ToString() + " |" + outText + "\r\n");
            }
            else
            {
                textBox1.AppendText("no barcodes node\r\n");
            }

            return nodeBarcodes;
        }

        private void DrawResult(Image img, XmlNode nodeBarcodes)
        {
            Graphics gr = Graphics.FromImage(img);// pictureBox1.CreateGraphics();

            if (nodeBarcodes == null)
                return;
            int barNum = Int32.Parse(nodeBarcodes.Attributes["qty"]?.InnerText);
            for (int i = 0; i < barNum; ++i)
            {
                XmlNode nodeBarcode = nodeBarcodes.ChildNodes.Item(i);
                Debug.Assert(nodeBarcode != null && nodeBarcode.Name == "barcode");
                XmlNode nodeCoord = nodeBarcode.SelectSingleNode("coordinates");
                PointF[] points = new PointF[5];
                points[0].X = Int32.Parse(nodeCoord.Attributes["x1"]?.InnerText);
                points[0].Y = Int32.Parse(nodeCoord.Attributes["y1"]?.InnerText);
                points[1].X = Int32.Parse(nodeCoord.Attributes["x3"]?.InnerText);
                points[1].Y = Int32.Parse(nodeCoord.Attributes["y3"]?.InnerText);
                points[2].X = Int32.Parse(nodeCoord.Attributes["x4"]?.InnerText);
                points[2].Y = Int32.Parse(nodeCoord.Attributes["y4"]?.InnerText);
                points[3].X = Int32.Parse(nodeCoord.Attributes["x2"]?.InnerText);
                points[3].Y = Int32.Parse(nodeCoord.Attributes["y2"]?.InnerText);
                points[4] = points[0];

                SolidBrush fillBrush = new SolidBrush(Color.FromArgb(140, 230, 230, 255));
                gr.FillPolygon(fillBrush, points);

                Pen pen = new Pen(Color.FromArgb(0, 255, 0), 3);
                gr.DrawLines(pen, points);
            }

            pictureBox1.Image = img;
        }

        static async Task<string> Upload(string apiKey, string url, string sourceFile, uint barcodeTypes)
        {
            using (HttpClientHandler handler = new HttpClientHandler())
            using (HttpClient client = new HttpClient(handler))
            {
                var request = new MultipartFormDataContent();

                request.Add(new StringContent(apiKey), "key");
                request.Add(new StringContent("barcode-reader"), "target");
                request.Add(new StringContent("xml"), "out_format");
                request.Add(new StringContent("7"), "lnum");
                request.Add(new StringContent(barcodeTypes.ToString()), "types");
                request.Add(new StreamContent(File.OpenRead(sourceFile)), "source_file", new FileInfo(sourceFile).Name);

                using (HttpResponseMessage response = await client.PostAsync(url, request).ConfigureAwait(false))
                using (HttpContent content = response.Content)
                {
                    string data = await content.ReadAsStringAsync();
                    return data;
                }
            }
        }

        static async Task<string> Query(string key, string url)
        {
            using (HttpClientHandler handler = new HttpClientHandler())
            using (HttpClient client = new HttpClient())
            using (HttpResponseMessage response = await client.GetAsync(url).ConfigureAwait(continueOnCapturedContext: false))
            using (HttpContent content = response.Content)
            {
                string data = await content.ReadAsStringAsync();
                return data;
            }
        }

    }
}
